@@ -154,6 +154,9 @@ class Object(models.Model):
         
     def get_revision(self,rev):
         return self.vcs.get_revision(self.repo_path,rev)
+    
+    def annotate(self):
+        return self.vcs.annotate(self.repo_path)
         
     @classmethod
     def get_object_class(self,repo):
@@ -343,7 +346,7 @@ class Config(Object):
     ##
     ##
     def change_link(self):
-        return "<a href='%s'>Change</a>"%site.reverse("cm:config:change",self.id)
+        return "<a href='%s' class='changelink'>Change</a>"%site.reverse("cm:config:change",self.id)
     change_link.short_description="Change"
     change_link.allow_tags=True
 
