@@ -14,8 +14,8 @@ from noc.lib.fileutils import is_differ,rewrite_when_differ,safe_rewrite
 from noc.dns.generators import generator_registry
 from noc.lib.rpsl import rpsl_format
 from noc.lib.ip import generate_ips
-from noc.main.menu import Menu
-
+from noc.lib.fields import AutoCompleteTagsField
+from noc.lib.app.site import site
 ##
 ## register all generator classes
 ##
@@ -114,15 +114,18 @@ class DNSZone(models.Model):
     serial=models.CharField("Serial",max_length=10,default="0000000000")
     profile=models.ForeignKey(DNSZoneProfile,verbose_name="Profile")
     paid_till=models.DateField("Paid Till",null=True,blank=True)
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     
     # Managers
     objects=models.Manager()
     forward_zones=ForwardZoneManager()
     reverse_zones=ReverseZoneManager()
-    def __str__(self):
-        return self.name
     def __unicode__(self):
         return self.name
+    
+    def get_absolute_url(self):
+        return site.reverse("dns:dnszone:change",self.id)
+    
     def _type(self):
         if self.name.lower().endswith(".in-addr.arpa"):
             return "R"
@@ -266,7 +269,7 @@ class DNSZone(models.Model):
     distribution_list=property(_distribution_list)
     
     def distribution(self):
-        return ", ".join(["<A HREF='/dns/%s/zone/%s/'>%s</A>"%(self.name,n.id,n.name) for n in self.distribution_list])
+        return ", ".join(["<A HREF='/dns/dnszone/%d/ns/%d/'>%s</A>"%(self.id,n.id,n.name) for n in self.distribution_list])
     distribution.short_description="Distribution"
     distribution.allow_tags=True
 
@@ -304,11 +307,6 @@ class DNSZone(models.Model):
         s=["domain: %s"%self.name]+["nserver: %s"%ns for ns in self.ns_list]
         return rpsl_format("\n".join(s))
     rpsl=property(_rpsl)
-    
-    def rpsl_link(self):
-        return "<A HREF='/dns/%s/zone/rpsl/'>RPSL</A>"%self.name
-    rpsl_link.short_description="RPSL"
-    rpsl_link.allow_tags=True
 ##
 ##
 ##
@@ -359,21 +357,10 @@ class DNSZoneRecord(models.Model):
     left=models.CharField("Left",max_length=32,blank=True,null=True)
     type=models.ForeignKey(DNSZoneRecordType,verbose_name="Type")
     right=models.CharField("Right",max_length=64) #,core=True)
-    def __str__(self):
-        return "%s %s"%(self.zone.name," ".join([x for x in [self.left,self.type.type,self.right] if x is not None]))
+    tags=AutoCompleteTagsField("Tags",null=True,blank=True)
     def __unicode__(self):
-        return unicode(str(self))
-##
-## Application Menu
-##
-class AppMenu(Menu):
-    app="dns"
-    title="DNS"
-    items=[
-        ("Zones", "/admin/dns/dnszone/", "dns.change_dnszone"),
-        ("Setup",[
-            ("DNS Servers",      "/admin/dns/dnsserver/",         "dns.change_dnsserver"),
-            ("Zone Profiles",    "/admin/dns/dnszoneprofile/",    "dns.change_dnszoneprofile"),
-            ("Zone Record Types","/admin/dns/dnszonerecordtype/", "dns.change_dnszonerecordtype"),
-        ])
-    ]
+        return u"%s %s"%(self.zone.name," ".join([x for x in [self.left,self.type.type,self.right] if x is not None]))
+
+    def get_absolute_url(self):
+        return site.reverse("dns:dnszone:change",self.zone.id)
+        
