@@ -8,15 +8,11 @@
 from __future__ import with_statement
 from django.db import models
 from noc.sa.models import ManagedObject,ManagedObjectSelector
-from noc.main.models import TimePattern,NotificationGroup
+from noc.main.models import TimePattern,NotificationGroup,PyRule
 from noc.settings import config
 from noc.lib.fileutils import safe_rewrite
-from noc.fm.triggers import event_trigger_registry
-from noc.main.menu import Menu
-import imp,subprocess,tempfile,os,datetime,re
+import imp,subprocess,tempfile,os,datetime,re,random
 
-##
-event_trigger_registry.register_all()
 ##
 ## Python quote helper
 ##
@@ -310,22 +306,12 @@ class EventClass(models.Model):
     last_modified=models.DateTimeField("last_modified",auto_now=True)
     repeat_suppression=models.BooleanField("Repeat Suppression",default=False)
     repeat_suppression_interval=models.IntegerField("Repeat Suppression interval (secs)",default=3600)
-    trigger=models.CharField("Trigger",max_length=64,null=True,blank=True,choices=event_trigger_registry.choices)
+    rule=models.ForeignKey(PyRule,verbose_name="pyRule",null=True,blank=True,
+        limit_choices_to={"interface":"IEvent"})
     is_builtin=models.BooleanField("Is Builtin",default=False)
     
     def __unicode__(self):
         return self.name
-    
-    def python_link(self):
-        return "<A HREF='/fm/py_event_class/%d/'>Python</A>"%self.id
-    python_link.short_description="Python"
-    python_link.allow_tags=True
-    ##
-    ## Run trigger if defined
-    ##
-    def run_trigger(self,event):
-        if self.trigger and self.trigger in event_trigger_registry.classes:
-            event_trigger_registry.classes[self.trigger]().handle(event)
     ##
     ## Python representation of data structure
     ##
@@ -339,10 +325,8 @@ class EventClass(models.Model):
         s+=["    body_template=\"\"\"%s\"\"\""%self.body_template]
         s+=["    repeat_suppression=%s"%self.repeat_suppression]
         s+=["    repeat_suppression_interval=%d"%self.repeat_suppression_interval]
-        if self.trigger:
-            s+=["    trigger=\"%s\""%self.trigger]
-        else:
-            s+=["    trigger=None"]
+        if self.rule:
+            s+=["    rule=\"%s\""%self.rule.name]
         vars=list(self.eventclassvar_set.all())
         if vars:
             s+=["    class Vars:"]
@@ -374,19 +358,14 @@ class EventClassificationRule(models.Model):
         verbose_name="Event Classification Rule"
         verbose_name_plural="Event Classification Rules"
         ordering=["preference"]
-    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
     name=models.CharField("Name",max_length=64)
+    event_class=models.ForeignKey(EventClass,verbose_name="Event Class")
     preference=models.IntegerField("Preference",1000)
     action=models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A")
     is_builtin=models.BooleanField("Is Builtin",default=False)
     
     def __unicode__(self):
         return self.name
-        
-    def python_link(self):
-        return "<A HREF='/fm/py_event_classification_rule/%d/'>Python</A>"%self.id
-    python_link.short_description="Python"
-    python_link.allow_tags=True
     ##
     ## Python representation of data structure
     ##
@@ -408,6 +387,55 @@ class EventClassificationRule(models.Model):
         s+=["    ]",""]
         return "\n".join(s)
     python_code=property(_python_code)
+    ##
+    ## Return clone of existing rule
+    ##
+    def clone(self):
+        # Find suitable rule name
+        i=1
+        while True:
+            name=self.name+" copy #%d"%i
+            try:
+                EventClassificationRule.objects.get(name=name)
+            except EventClassificationRule.DoesNotExist:
+                break
+            i+=1
+        # Create cloned rule
+        new_rule=EventClassificationRule(
+            name=name,
+            event_class=self.event_class,
+            preference=self.preference,
+            is_builtin=False)
+        new_rule.save()
+        for r in self.eventclassificationre_set.all():
+            new_r=EventClassificationRE(rule=new_rule,left_re=r.left_re,right_re=r.right_re)
+            new_r.save()
+        return new_rule
+    ##
+    ## Return rule created from event
+    ##
+    @classmethod
+    def from_event(cls,event):
+        def re_q(s):
+            for qc in ["\\",".","+","*","[","]","(",")"]:
+                s=s.replace(qc,"\\"+qc)
+            return s
+        s=""
+        sources=list(event.eventdata_set.filter(key="source"))
+        if len(sources)>0:
+            source=sources[0].value
+            if "SNMP" in source:
+                s=" SNMP"
+            elif "syslog" in source:
+                s=" SYSLOG"
+        name="%s %d:%d%s"%(event.managed_object.profile_name,event.id,random.randint(0,100000),s)
+        rule=EventClassificationRule(event_class=event.event_class,
+            name=name,preference=1000)
+        rule.save()
+        for d in event.eventdata_set.filter(type__in=[">","R"]):
+            r=EventClassificationRE(rule=rule,left_re="^%s$"%re_q(d.key)[:254],right_re="^%s$"%re_q(d.value)[:254])
+            r.save()
+        return rule
 ##
 ## Regular expressions to match event vars
 ##
@@ -445,8 +473,52 @@ class EventPostProcessingRule(models.Model):
     change_category=models.ForeignKey(EventCategory,verbose_name="Change Category to",blank=True,null=True)
     action=models.CharField("Action",max_length=1,choices=[("A","Make Active"),("C","Close"),("D","Drop")],default="A")
     notification_group=models.ForeignKey(NotificationGroup,verbose_name="Notification Group",null=True,blank=True)
+    rule=models.ForeignKey(PyRule,verbose_name="pyRule",null=True,blank=True,
+        limit_choices_to={"interface":"IEvent"})
     def __unicode__(self):
         return self.name
+    ##
+    ## Return clone of existing rule
+    ##
+    def clone(self):
+        # Find clone name
+        i=1
+        while True:
+            name=self.name+" copy #%d"%i
+            try:
+                EventPostProcessingRule.objects.get(name=name)
+            except EventPostProcessingRule.DoesNotExist:
+                break
+            i+=1
+        # Create cloned rule
+        new_rule=EventPostProcessingRule(
+            name=name,
+            event_class=self.event_class,
+            preference=self.preference,
+            description=self.description,
+            change_priority=self.change_priority,
+            change_category=self.change_category,
+            action=self.action,
+            is_active=True)
+        new_rule.save()
+        # Copy RE
+        for r in self.eventpostprocessingre_set.all():
+            new_r=EventPostProcessingRE(rule=new_rule,var_re=r.var_re,value_re=r.value_re)
+            new_r.save()
+        return new_rule
+    ##
+    ## Return rule created from event
+    ##
+    @classmethod
+    def from_event(cls,event):
+        def re_q(s):
+            return s.replace("\\","\\\\").replace(".","\\.").replace("+","\\+").replace("*","\\*")
+        rule=EventPostProcessingRule(event_class=event.event_class,name="Rule #%d:%d"%(event.id,random.randint(0,100000)),preference=1000)
+        rule.save()
+        for d in event.eventdata_set.filter(type="V"):
+            r=EventPostProcessingRE(rule=rule,var_re="^%s$"%re_q(d.key)[:254],value_re="^%s$"%re_q(d.value)[:254])
+            r.save()
+        return rule
 ##
 ## Regular expressions to match event vars
 ##
@@ -476,10 +548,6 @@ class EventCorrelationRule(models.Model):
     def __unicode__(self):
         return self.name
         
-    def python_link(self):
-        return "<A HREF='/fm/py_event_correlation_rule/%d/'>Python</A>"%self.id
-    python_link.short_description="Python"
-    python_link.allow_tags=True
     ##
     ## Python representation of data structure
     ##
@@ -657,23 +725,17 @@ class EventArchivationRule(models.Model):
         return self.ttl*{"s":1,"m":60,"h":3600,"d":86400}[self.ttl_measure]
     ttl_seconds=property(_ttl_seconds)
 ##
-## Application Menu
 ##
-class AppMenu(Menu):
-    app="fm"
-    title="Fault Management"
-    items=[
-        ("Events", "/fm/", "fm.change_event"),
-        ("Active Problems", "/fm/active_problems_summary/", "fm.change_event"),
-        ("Setup", [
-            ("MIBs",                "/admin/fm/mib/",                     "fm.change_mib"),
-            ("MIB Data",            "/admin/fm/mibdata/",                 "fm.change_mibdata"),
-            ("Event Classes",       "/admin/fm/eventclass/",              "fm.change_eventclass"),
-            ("Event Categories",    "/admin/fm/eventcategory/",           "fm.change_eventcategory"),
-            ("Event Priorities",    "/admin/fm/eventpriority/",           "fm.change_eventpriority"),
-            ("Classification Rules","/admin/fm/eventclassificationrule/", "fm.change_eventclassificationrule"),
-            ("Post-Processing Rules", "/admin/fm/eventpostprocessingrule/", "fm.change_eventpostprocessingrule"),
-            ("Correlation Rules",   "/admin/fm/eventcorrelationrule/",    "fm.change_eventcorrelationrule"),
-            ("Archivation Rules",   "/admin/fm/eventarchivationrule/",    "fm.change_eventarchivationrule"),
-        ])
-    ]
+##
+class IgnoreEventRules(models.Model):
+    class Meta:
+        verbose_name="Ignore Event Rule"
+        verbose_name_plural="Ignore Event Rules"
+        unique_together=[("left_re","right_re")]
+    name=models.CharField("Name",max_length=64,unique=True)
+    left_re=models.CharField("Left RE",max_length=256)
+    right_re=models.CharField("Right Re",max_length=256)
+    is_active=models.BooleanField("Is Active",default=True)
+    description=models.TextField("Description",null=True,blank=True)
+    def __unicode__(self):
+        return "%s (%s,%s)"%(self.name,self.left_re,self.right_re)
