@@ -13,7 +13,10 @@ from noc.vc.models import VC,VCBindFilter
 from noc.lib.colors import get_colors
 from noc.lib.validators import is_cidr,is_ipv4,is_fqdn
 from noc.lib.ip import normalize_prefix,contains,in_range,free_blocks,prefix_to_size
-from noc.lib.widgets import AutoCompleteTags
+from noc.lib.widgets import AutoCompleteTags,AutoCompleteTextInput
+from noc.lib.forms import NOCForm
+from noc.sa.interfaces.base import MACAddressParameter,InterfaceTypeError
+from noc.sa.models import ManagedObject,ReduceTask
 ##
 ## IP Address Space Management
 ##
@@ -23,6 +26,11 @@ class IPManageAppplication(Application):
     ## Display VRF list
     ##
     def view_index(self,request):
+        # Check only one active VRF exists
+        if VRF.objects.count()==1:
+            vrf=VRF.objects.all()[0]
+            return self.response_redirect("ip:ipmanage:vrf_index",vrf.id,"0.0.0.0/0")
+        #
         groups=[]
         for vg in VRFGroup.objects.all():
             vrfs=vg.vrf_set.all()
@@ -56,6 +64,7 @@ class IPManageAppplication(Application):
         ranges=None
         range_colors=None
         all_addresses=None
+        can_ping=False
         if has_children:
             orphaned_addresses=prefix.orphaned_addresses
         else:
@@ -74,6 +83,8 @@ class IPManageAppplication(Application):
             for a in prefix.all_addresses:
                 try:
                     ip=a.ip
+                    if a.managed_object:
+                        can_ping=True
                 except:
                     ip=a
                 ar=None
@@ -86,7 +97,7 @@ class IPManageAppplication(Application):
                             "can_allocate":can_allocate,"block_info":block_info,"has_children":has_children,
                             "ranges":ranges,"range_colors":range_colors,"all_addresses":all_addresses,
                             "orphaned_addresses":orphaned_addresses,
-                            "has_bookmark":prefix.has_bookmark(request.user),
+                            "has_bookmark":prefix.has_bookmark(request.user),"can_ping":can_ping,
                             "my_networks":IPv4BlockBookmark.user_bookmarks(request.user,vrf)})
     view_vrf_index.url=r"(?P<vrf_id>\d+)/(?P<prefix>\S+)/$"
     view_vrf_index.url_name="vrf_index"
@@ -121,14 +132,14 @@ class IPManageAppplication(Application):
     ##
     ## Allocate new block form
     ##
-    class AllocateBlockForm(forms.Form):
-        prefix=forms.CharField(label="prefix",required=True)
-        description=forms.CharField(label="description",required=True)
+    class AllocateBlockForm(NOCForm):
+        prefix=forms.CharField(label="Prefix",required=True)
+        description=forms.CharField(label="Description",required=True)
         asn=forms.ModelChoiceField(label="ASN",queryset=AS.objects.all(),required=True)
         tags=forms.CharField(widget=AutoCompleteTags,required=False)
         tt=forms.IntegerField(label="TT #",required=False)
         def __init__(self,data=None,initial=None,vrf=None,block_id=None):
-            forms.Form.__init__(self,data=data,initial=initial)
+            NOCForm.__init__(self,data=data,initial=initial)
             self.vrf=vrf
             self.block_id=block_id
         def clean_prefix(self):
@@ -210,6 +221,12 @@ class IPManageAppplication(Application):
                     status="created"
                 block.save()
                 self.message_user(request,"Block '%s' in VRF '%s' %s successfully"%(block.prefix,str(vrf),status))
+                # Redirect depenging on submit button pressed
+                if "_continue" in request.POST:
+                    return self.response_redirect("ip:ipmanage:change_block",vrf.id,block.prefix)
+                if "_addanother" in request.POST:
+                    return self.response_redirect("ip:ipmanage:allocate_block",vrf.id)
+                
                 return self.response_redirect("%s%d/%s/"%(self.base_url,vrf.id,block.prefix))
         else:
             # Display form
@@ -242,14 +259,18 @@ class IPManageAppplication(Application):
     ##
     ## Assign new IP address form
     ##
-    class AssignAddressForm(forms.Form):
+    class AssignAddressForm(NOCForm):
         fqdn=forms.CharField(label="FQDN",required=True)
         ip=forms.CharField(label="IP",required=True)
+        mac=forms.CharField(label="MAC",required=False)
+        auto_update_mac=forms.BooleanField(label="Auto Update MAC",required=False)
+        managed_object=forms.CharField(label="Managed Object",required=False,widget=AutoCompleteTextInput("sa:managedobject:lookup"))
         description=forms.CharField(label="Description",required=False)
-        tags=forms.CharField(widget=AutoCompleteTags,required=False)
+        tags=forms.CharField(label="Tags",widget=AutoCompleteTags,required=False)
         tt=forms.IntegerField(label="TT #",required=False)
         def __init__(self,data=None,initial=None,vrf=None,address_id=None):
-            forms.Form.__init__(self,data=data,initial=initial)
+            NOCForm.__init__(self,data=data,initial=initial)
+            #forms.Form.__init__(self,data=data,initial=initial)
             self.vrf=vrf
             self.address_id=address_id
         def clean_fqdn(self):
@@ -273,6 +294,13 @@ class IPManageAppplication(Application):
             if q.count()>0:
                 raise forms.ValidationError("IPv4 Address is already present")
             return ip
+        def clean_mac(self):
+            if not self.cleaned_data["mac"]:
+                return ""
+            try:
+                return MACAddressParameter().clean(self.cleaned_data["mac"])
+            except InterfaceTypeError:
+                raise forms.ValidationError("Invalid MAC address")
     ##
     ## Assign IP address
     ##
@@ -287,12 +315,16 @@ class IPManageAppplication(Application):
             address=get_object_or_404(IPv4Address,vrf=vrf,ip=ip)
             address_id=address.id
             initial={
-                "fqdn"        : address.fqdn,
-                "ip"          : address.ip,
-                "description" : address.description,
-                "tags"        : address.tags,
-                "tt"          : address.tt,
+                "fqdn"            : address.fqdn,
+                "ip"              : address.ip,
+                "mac"             : address.mac,
+                "auto_update_mac" : address.auto_update_mac,
+                "description"     : address.description,
+                "tags"            : address.tags,
+                "tt"              : address.tt,
             }
+            if address.managed_object:
+                initial["managed_object"]=address.managed_object.name
             p="/"+ip
             parents=list(address.parent.parents)+[address.parent]
             can_delete=True
@@ -308,9 +340,15 @@ class IPManageAppplication(Application):
             if form.is_valid():
                 if not IPv4BlockAccess.check_write_access(request.user,vrf,form.cleaned_data["ip"]+"/32"):
                     return self.response_forbidden("Permission denied")
+                managed_object=None
+                if "managed_object" in form.cleaned_data and form.cleaned_data["managed_object"]:
+                    managed_object=get_object_or_404(ManagedObject,name=form.cleaned_data["managed_object"])
                 if ip:
                     address.fqdn=form.cleaned_data["fqdn"]
                     address.ip=form.cleaned_data["ip"]
+                    address.mac=form.cleaned_data["mac"]
+                    address.auto_update_mac="auto_update_mac" in form.cleaned_data and form.cleaned_data["auto_update_mac"]
+                    address.managed_object=managed_object
                     address.description=form.cleaned_data["description"]
                     address.tags=form.cleaned_data["tags"]
                     address.tt=form.cleaned_data["tt"]
@@ -320,11 +358,18 @@ class IPManageAppplication(Application):
                     if IPv4Address.objects.filter(vrf=vrf,ip=form.cleaned_data["ip"]).count()>0:
                         return render_failure(request,"Duplicated IP address","Address %s is already present in VRF %s"%(form.cleaned_data["ip"],vrf.name))
                     address=IPv4Address(vrf=vrf,fqdn=form.cleaned_data["fqdn"],
-                        ip=form.cleaned_data["ip"],description=form.cleaned_data["description"],
+                        ip=form.cleaned_data["ip"],mac=form.cleaned_data["mac"],
+                        auto_update_mac="auto_update_mac" in form.cleaned_data and form.cleaned_data["auto_update_mac"],
+                        managed_object=managed_object,description=form.cleaned_data["description"],
                         tags=form.cleaned_data["tags"],tt=form.cleaned_data["tt"])
                     status="created"
                 address.save()
                 self.message_user(request,"IP Address %s in VRF %s %s successfully"%(form.cleaned_data["ip"],str(vrf),status))
+                # Redirect depenging on submit button pressed
+                if "_continue" in request.POST:
+                    return self.response_redirect("ip:ipmanage:change_address",vrf.id,address.ip)
+                if "_addanother" in request.POST:
+                    return self.response_redirect("ip:ipmanage:assign_address",vrf.id)
                 return self.response_redirect("ip:ipmanage:vrf_index",vrf.id,address.parent.prefix)
         else:
             if "ip" not in initial:
@@ -407,6 +452,45 @@ class IPManageAppplication(Application):
     view_toogle_bookmark.url_name="toggle_bookmark"
     view_toogle_bookmark.access=HasPerm("view")
     ##
+    ## AJAX handler to run ping_task
+    ##
+    def view_ping_check(self,request,vrf_id,prefix):
+        vrf=get_object_or_404(VRF,id=int(vrf_id))
+        p=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
+        # Detect managed objects in block
+        addresses=p.addresses
+        r=[a for a in addresses if a.managed_object]
+        if not r:
+            return self.render_json(None)
+        activator_name=r[0].managed_object.activator.name
+        addresses=list(generate_ips(p.network,p.broadcast))
+        if len(addresses)>2:
+            addresses=addresses[1:-1]
+        t=ReduceTask.create_task(["SAE"],"pyrule:get_single_result",{},
+            "ping_check",{"activator_name":activator_name,"addresses":addresses},60)
+        return self.render_json(t.id)
+    view_ping_check.url=r"^(?P<vrf_id>\d+)/(?P<prefix>\S+)/ping_check/$"
+    view_ping_check.url_name="ping_check"
+    view_ping_check.access=HasPerm("view")
+    ##
+    ## Ping check task result
+    ##
+    def view_ping_check_task(self,request,vrf_id,prefix,task_id):
+        vrf=get_object_or_404(VRF,id=int(vrf_id))
+        p=get_object_or_404(IPv4Block,vrf=vrf,prefix=prefix)
+        task=get_object_or_404(ReduceTask,id=int(task_id))
+        try:
+            result=task.get_result(block=False)
+        except ReduceTask.NotReady:
+            return self.render_json(None) # Waiting
+        r={}
+        for s in result:
+            r[s["ip"]]=s["status"]
+        return self.render_json(r)
+    view_ping_check_task.url=r"^(?P<vrf_id>\d+)/(?P<prefix>\S+)/ping_check/(?P<task_id>\d+)/$"
+    view_ping_check_task.url_name="ping_check_task"
+    view_ping_check_task.access=HasPerm("view")
+    ##
     ## Return a list of user access
     ##
     def user_access_list(self,user):
