@@ -8,9 +8,11 @@
 """
 """
 import re,types,datetime
+from noc.lib.text import list_to_ranges,ranges_to_list,list_to_ranges
 
 try:
     from django import forms
+    from noc.lib.forms import NOCForm
 except ImportError:
     # No django. Interface.get_form() is meaningless
     pass
@@ -66,8 +68,8 @@ class Parameter(object):
     ##
     ## Returns an form field object
     ##
-    def get_form_field(self):
-        return forms.CharField(required=self.required,initial=self.default)
+    def get_form_field(self,label=None):
+        return forms.CharField(required=self.required,initial=self.default,label=label)
 ##
 ##
 ##
@@ -77,7 +79,7 @@ class ORParameter(Parameter):
     10
     >>> ORParameter(IntParameter(),IPParameter()).clean("192.168.1.1")
     '192.168.1.1'
-    >>> ORParameter(IntParameter(),IPParameter()).clean("xxx")
+    >>> ORParameter(IntParameter(),IPParameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: 'xxx'
@@ -85,17 +87,29 @@ class ORParameter(Parameter):
     10
     >>> (IntParameter()|IPParameter()).clean("192.168.1.1")
     '192.168.1.1'
-    >>> (IntParameter()|IPParameter()).clean("xxx")
+    >>> (IntParameter()|IPParameter()).clean("xxx") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: 'xxx'
+    >>> (IntParameter()|IPParameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+        ...
+    InterfaceTypeError: IPParameter: None.
+    >>> (IntParameter(required=False)|IPParameter(required=False)).clean(None)
+    >>> (IntParameter(required=False)|IPParameter()).clean(None) #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+        ...
+    InterfaceTypeError: IPParameter: None.
     """
     def __init__(self,left,right):
         super(ORParameter,self).__init__()
         self.left=left
         self.right=right
+        self.required=self.left.required or self.right.required
         
     def clean(self,value):
+        if value is None and self.required==False:
+            return None
         try:
             return self.left.clean(value)
         except InterfaceTypeError:
@@ -118,7 +132,7 @@ class ORParameter(Parameter):
 class NoneParameter(Parameter):
     """
     >>> NoneParameter().clean(None)
-    >>> NoneParameter().clean("None")
+    >>> NoneParameter().clean("None") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: NoneParameter: 'None'
@@ -144,14 +158,27 @@ class StringParameter(Parameter):
     'no test'
     >>> StringParameter(default="test").clean(None)
     'test'
+    >>> StringParameter(choices=["1","2"]).clean("1")
+    '1'
+    >>> StringParameter(choices=["1","2"]).clean("3") #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: StringParameter: '3'.
     """
+    def __init__(self,required=True,default=None,choices=None):
+        self.choices=choices
+        super(StringParameter,self).__init__(required=required,default=default)
+        
     def clean(self,value):
         if value is None and self.default is not None:
             return self.default
         try:
-            return str(value)
+            value=str(value)
         except:
             self.raise_error(value)
+        if self.choices and value not in self.choices:
+            self.raise_error(value)
+        return value
 ##
 ##
 ##
@@ -163,7 +190,7 @@ class REStringParameter(StringParameter):
     'exxp'
     >>> REStringParameter("ex+p").clean("regexp 1")
     'regexp 1'
-    >>> REStringParameter("ex+p").clean("ex")
+    >>> REStringParameter("ex+p").clean("ex") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: REStringParameter: 'ex'
@@ -200,10 +227,10 @@ class BooleanParameter(Parameter):
     True
     >>> BooleanParameter().clean(0)
     False
-    >>> BooleanParameter().clean([])
+    >>> BooleanParameter().clean([]) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
-        ...
-    InterfaceTypeError: BooleanParameter: []
+    ...
+    InterfaceTypeError: BooleanParameter: [].
     >>> BooleanParameter(default=False).clean(None)
     False
     >>> BooleanParameter(default=True).clean(None)
@@ -220,8 +247,8 @@ class BooleanParameter(Parameter):
             return value.lower() in ["true","t","yes","y"]
         self.raise_error(value)
     ##
-    def get_form_field(self):
-        return forms.BooleanField(required=self.required,initial=self.default)
+    def get_form_field(self,label=None):
+        return forms.BooleanField(required=self.required,initial=self.default,label=label)
     
 ##
 ##
@@ -232,15 +259,15 @@ class IntParameter(Parameter):
     1
     >>> IntParameter().clean("1")
     1
-    >>> IntParameter().clean("not a number")
+    >>> IntParameter().clean("not a number") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IntParameter: 'not a number'
-    >>> IntParameter(min_value=10).clean(5)
+    >>> IntParameter(min_value=10).clean(5) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IntParameter: 5
-    >>> IntParameter(max_value=7).clean(10)
+    >>> IntParameter(max_value=7).clean(10) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IntParameter: 10
@@ -248,7 +275,7 @@ class IntParameter(Parameter):
     5
     >>> IntParameter(max_value=10,default=7).clean(None)
     7
-    >>> IntParameter(max_value=10,default=15)
+    >>> IntParameter(max_value=10,default=15) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IntParameter: 15
@@ -297,13 +324,13 @@ class InstanceOfParameter(Parameter):
     'Ok'
     >>> InstanceOfParameter(cls=C).clean(CC()) and "Ok"
     'Ok'
-    >>> InstanceOfParameter(cls=C).clean(1) and "Ok"
+    >>> InstanceOfParameter(cls=C).clean(1) and "Ok" #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     InterfaceTypeError: InstanceOfParameter: 1
     >>> InstanceOfParameter(cls="C").clean(C()) and "Ok"
     'Ok'
-    >>> InstanceOfParameter(cls="C").clean(1) and "Ok"
+    >>> InstanceOfParameter(cls="C").clean(1) and "Ok" #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     InterfaceTypeError: InstanceOfParameter: 1
@@ -323,8 +350,11 @@ class InstanceOfParameter(Parameter):
         return hasattr(value,"__class__") and value.__class__.__name__==self.cls
         
     def clean(self,value):
-        if value is None and self.default is not None:
-            return self.default
+        if value is None:
+            if self.default is not None:
+                return self.default
+            if not self.required:
+                return None
         try:
             if self.is_valid(value):
                 return value
@@ -334,13 +364,78 @@ class InstanceOfParameter(Parameter):
 ##
 ##
 ##
+class SubclassOfParameter(Parameter):
+    """
+    >>> class C: pass
+    >>> class C1(C): pass
+    >>> class C2(C1): pass
+    >>> class C3(C1): pass
+    >>> SubclassOfParameter(cls=C).clean(C2) and "Ok"
+    'Ok'
+    >>> SubclassOfParameter(cls=C).clean(1) #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: SubclassOfParameter: 1
+    >>> SubclassOfParameter(cls="C").clean(C2) and "Ok"
+    'Ok'
+    >>> SubclassOfParameter(cls=C).clean(1) #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: SubclassOfParameter: 1
+    >>> SubclassOfParameter(cls=C2).clean(C3) #doctest: +IGNORE_EXCEPTION_DETAIL
+    Traceback (most recent call last):
+    ...
+    InterfaceTypeError: SubclassOfParameter: <class base.C3>
+    >>> SubclassOfParameter(cls="C",required=False).clean(None)
+    """
+    def __init__(self,cls,required=True,default=None):
+        super(SubclassOfParameter,self).__init__(required=required,default=default)
+        self.cls=cls
+        if isinstance(cls,basestring):
+            self.is_valid=self.is_valid_classname
+        else:
+            self.is_valid=self.is_valid_class
+    
+    def is_valid_classname(self,value):
+        def check_name(c,name):
+            # Check class name
+            if c.__name__==name:
+                return True
+            # Check base classes name
+            for bc in c.__bases__:
+                if check_name(bc,name):
+                    return True
+            #
+            return False
+        
+        return check_name(value,self.cls)
+    
+    def is_valid_class(self,value):
+        return issubclass(value,self.cls)
+    
+    def clean(self,value):
+        if value is None:
+            if self.default is not None:
+                return self.default
+            if not self.required:
+                return None
+        try:
+            if self.is_valid(value):
+                return value
+        except:
+            pass
+        self.raise_error(value)
+        
+##
+##
+##
 class ListOfParameter(ListParameter):
     """
     >>> ListOfParameter(element=IntParameter()).clean([1,2,3])
     [1, 2, 3]
     >>> ListOfParameter(element=IntParameter()).clean([1,2,"3"])
     [1, 2, 3]
-    >>> ListOfParameter(element=IntParameter()).clean([1,2,"x"])
+    >>> ListOfParameter(element=IntParameter()).clean([1,2,"x"]) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
     ...
     InterfaceTypeError: IntParameter: 'x'
@@ -392,7 +487,7 @@ class DictParameter(Parameter):
     """
     >>> DictParameter(attrs={"i":IntParameter(),"s":StringParameter()}).clean({"i":10,"s":"ten"})
     {'i': 10, 's': 'ten'}
-    >>> DictParameter(attrs={"i":IntParameter(),"s":StringParameter()}).clean({"i":"10","x":"ten"})
+    >>> DictParameter(attrs={"i":IntParameter(),"s":StringParameter()}).clean({"i":"10","x":"ten"}) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: DictParameter: {'i': '10', 'x': 'ten'}
@@ -488,7 +583,7 @@ class IPParameter(StringParameter):
     """
     >>> IPParameter().clean("192.168.0.1")
     '192.168.0.1'
-    >>> IPParameter().clean("192.168.0.256")
+    >>> IPParameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: '192.168.0.256'
@@ -513,15 +608,15 @@ class IPv4PrefixParameter(StringParameter):
     """
     >>> IPv4PrefixParameter().clean("192.168.0.0/16")
     '192.168.0.0/16'
-    >>> IPParameter().clean("192.168.0.256")
+    >>> IPParameter().clean("192.168.0.256") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: '192.168.0.256'
-    >>> IPParameter().clean("192.168.0.0/33")
+    >>> IPParameter().clean("192.168.0.0/33") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: '192.168.0.0/33'
-    >>> IPParameter().clean("192.168.0.0/-5")
+    >>> IPParameter().clean("192.168.0.0/-5") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: IPParameter: '192.168.0.0/-5'
@@ -556,21 +651,49 @@ class VLANIDParameter(IntParameter):
     """
     >>> VLANIDParameter().clean(10)
     10
-    >>> VLANIDParameter().clean(5000)
+    >>> VLANIDParameter().clean(5000) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: VLANIDParameter: 5000
-    >>> VLANIDParameter().clean(0)
+    >>> VLANIDParameter().clean(0) #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: VLANIDParameter: 0
     """
     def __init__(self,required=True,default=None):
         super(VLANIDParameter,self).__init__(required=required,default=default,min_value=1,max_value=4095)
+
 ##
 ##
 ##
-rx_mac_address_cisco=re.compile("^[0-9A-F]{4}\.[0-9A-F]{4}\.[0-9A-F]{4}$")
+class VLANIDListParameter(ListOfParameter):
+    """
+    >>> VLANIDListParameter().clean(["1","2","3"])
+    [1, 2, 3]
+    >>> VLANIDListParameter().clean([1,2,3])
+    [1, 2, 3]
+    """
+    def __init__(self,required=True,default=None):
+        super(VLANIDListParameter,self).__init__(element=VLANIDParameter(),required=required,default=default)
+
+##
+##
+##
+class VLANIDMapParameter(StringParameter):
+    def clean(self,value):
+        """
+        >>> VLANIDMapParameter().clean("1,2,5-10")
+        '1-2,5-10'
+        """
+        vp=VLANIDParameter()
+        try:
+            return list_to_ranges([vp.clean(v) for v in ranges_to_list(value)])
+        except SyntaxError:
+            self.raise_error(value)
+##
+##
+##
+rx_mac_address_cisco=re.compile(r"^[0-9A-F]{4}(?P<sep>[.\-])[0-9A-F]{4}(?P=sep)[0-9A-F]{4}$")
 rx_mac_address_cisco_media=re.compile("^01[0-9A-F]{2}\.[0-9A-F]{4}\.[0-9A-F]{4}\.[0-9A-F]{2}$")
 rx_mac_address_sixblock=re.compile("^([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2}):([0-9A-F]{1,2})$")
 rx_mac_address_hp=re.compile("^[0-9A-F]{6}-[0-9A-F]{6}$")
@@ -582,7 +705,7 @@ class MACAddressParameter(StringParameter):
     '12:34:56:78:9A:BC'
     >>> MACAddressParameter().clean("0112.3456.789a.bc")
     '12:34:56:78:9A:BC'
-    >>> MACAddressParameter().clean("1234.5678.9abc.def0")
+    >>> MACAddressParameter().clean("1234.5678.9abc.def0") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: MACAddressParameter: '1234.5678.9ABC.DEF0'
@@ -594,14 +717,16 @@ class MACAddressParameter(StringParameter):
     '00:13:46:50:87:05'
     >>> MACAddressParameter().clean("123456-789abc")
     '12:34:56:78:9A:BC'
-    >>> MACAddressParameter().clean("12-34-56-78-9A-BC-DE")
+    >>> MACAddressParameter().clean("12-34-56-78-9A-BC-DE") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: MACAddressParameter: '12:34:56:78:9A:BC:DE'
-    >>> MACAddressParameter().clean("AB-CD-EF-GH-HJ-KL")
+    >>> MACAddressParameter().clean("AB-CD-EF-GH-HJ-KL") #doctest: +IGNORE_EXCEPTION_DETAIL
     Traceback (most recent call last):
         ...
     InterfaceTypeError: MACAddressParameter: 'AB:CD:EF:GH:HJ:KL'
+    >>> MACAddressParameter().clean("aabb-ccdd-eeff")
+    'AA:BB:CC:DD:EE:FF'
     """
     def clean(self,value):
         if value is None and self.default is not None:
@@ -614,7 +739,7 @@ class MACAddressParameter(StringParameter):
             return "%s:%s:%s:%s:%s:%s"%(value[:2],value[2:4],value[4:6],value[6:8],value[8:10],value[10:])
         match=rx_mac_address_cisco.match(value)
         if match:
-            value=value.replace(".","")
+            value=value.replace(".","").replace("-","")
         else:
             match=rx_mac_address_hp.match(value)
             if match:
@@ -725,9 +850,9 @@ class Interface(object):
                 except InterfaceTypeError:
                     raise forms.ValidationError("Invalid value")
             return lambda: clean_field_wrapper(form,name,param)
-        f=forms.Form(data)
+        f=NOCForm(data)
         for n,p in self.gen_parameters():
-            f.fields[n]=p.get_form_field()
+            f.fields[n]=p.get_form_field(n)
             setattr(f,"clean_%s"%n,get_clean_field_wrapper(f,n,p))
         return f
 ##
