@@ -9,12 +9,16 @@
 """
 from base import *
 from igetmacaddresstable import IGetMACAddressTable
+from igetarp import IGetARP
 from igetlldpneighbors import IGetLLDPNeighbors
+from igetspanningtree import IGetSpanningTree
 from igetportchannel import IGetPortchannel
 
 class IGetTopologyData(Interface):
     get_mac=BooleanParameter(required=False,default=False)  # Get MAC address table
+    get_arp=BooleanParameter(required=False,default=False)  # Get ARP cache
     get_lldp=BooleanParameter(required=False,default=False) # Get LLDP neighbor information
+    get_stp=BooleanParameter(required=False,default=False)  # Get STP information
     
     returns=DictParameter(attrs={
         # Set to true if "mac" is not empty
@@ -22,6 +26,11 @@ class IGetTopologyData(Interface):
         # Mac address table.
         # Returned only if get_mac is True
         "mac"        : IGetMACAddressTable.returns,
+        # Set to true if "arp" is not empy
+        "has_arp"    : BooleanParameter(required=False,default=False),
+        # ARP Cache
+        # Returned only if get_arp is True
+        "arp"        : IGetARP.returns,
         # Set to true if "lldp_neighbors" is not empty
         "has_lldp"   : BooleanParameter(required=False,default=False),
         # Chassis ID, returned only if get_lldp is set
@@ -29,6 +38,10 @@ class IGetTopologyData(Interface):
         # LLDP neighbors
         # Returned only if get_lldp is set
         "lldp_neighbors" : IGetLLDPNeighbors.returns,
+        # Set to true if "stp" is not empty
+        "has_stp"    : BooleanParameter(required=False,default=False),
+        # STP Protocol data
+        "stp"        : IGetSpanningTree.returns | NoneParameter(),
         # Portchannels
         "portchannels"   : IGetPortchannel.returns,
     })
