@@ -16,12 +16,13 @@ class Profile(noc.sa.profiles.Profile):
     supported_schemes=[TELNET,SSH]
     pattern_more="^ --More--"
     pattern_unpriveleged_prompt=r"^\S+?>"
+    pattern_syntax_error=r"% Invalid input detected at"
     command_disable_pager="terminal length 0"
     command_super="enable"
     command_enter_config="configure terminal"
     command_leave_config="exit"
     command_save_config="copy running-config startup-config\n"
-    pattern_prompt=r"^\S+?#"
+    pattern_prompt=r"^(?P<hostname>\S+?)#"
     requires_netmask_conversion=True
     convert_mac=noc.sa.profiles.Profile.convert_mac_to_cisco
     convert_interface_name=noc.sa.profiles.Profile.convert_interface_name_cisco
