@@ -20,8 +20,16 @@ class Script(noc.sa.script.Script):
         match_sys=rx_sys.search(v)
         v=self.cli("show version")
         match_ver=rx_ver.search(v.replace(":",""))
+        sub=match_ver.group("version").split(".")
+        if sub[0]=="1":
+            platform="ES3526XA-V2"
+        elif sub[0]=="2" and sub[1]=="3":
+            if int(sub[2])&1==1:
+                platform="ES3526XA-38"
+            else:
+                platform="ES3526XA-1-SL-38"
         return {
             "vendor"    : "EdgeCore",
-            "platform"  : match_sys.group("platform"),
+            "platform"  : platform,
             "version"   : match_ver.group("version"),
         }
