@@ -36,6 +36,8 @@ scheme_id={
 }
 ##
 class TimeOutError(Exception): pass
+class LoginError(Exception): pass
+class CLISyntaxError(Exception): pass
 ##
 ##
 ##
@@ -130,6 +132,9 @@ class Script(threading.Thread):
     SSH=scheme_id["ssh"]
     HTTP=scheme_id["http"]
     TIMEOUT=120 # 2min by default
+    #
+    LoginError=LoginError
+    CLISyntaxError=CLISyntaxError
 
     def __init__(self,profile,activator,access_profile,parent=None,**kwargs):
         self.start_time=time.time()
@@ -156,6 +161,7 @@ class Script(threading.Thread):
         self.status=False
         self.result=None
         self.error_traceback=None
+        self.login_error=None
         self.strip_echo=True
         self.kwargs=kwargs
         self.scripts=ScriptProxy(self)
@@ -224,13 +230,18 @@ class Script(threading.Thread):
         
     def run(self):
         self.debug("Running")
+        result=None
         try:
-            self.result=self.serialize_result(self.guarded_run())
+            result=self.guarded_run()
+            self.result=self.serialize_result(result)
             if self.parent is None and self.need_to_save and self.profile.command_save_config:
                 self.debug("Saving config")
                 self.cli(self.profile.command_save_config)
         except TimeOutError:
             self.error("Timed out")
+        except self.LoginError,why:
+            self.login_error=why.args[0]
+            self.error("Login failed: %s"%self.login_error)
         except:
             t,v,tb=sys.exc_info()
             r=[str(t),str(v)]
@@ -238,6 +249,8 @@ class Script(threading.Thread):
             self.error_traceback="\n".join(r)
             self.debug("Script traceback:\n%s"%self.error_traceback)
         self.debug("Closing")
+        if self.activator.to_save_output and result:
+            self.activator.save_result(result,self.motd)
         if self.cli_provider:
             self.activator.request_call(self.cli_provider.close)
         if self.snmp:
@@ -281,16 +294,33 @@ class Script(threading.Thread):
                 self.to_disable_pager=False
                 self.cli(self.profile.command_disable_pager)
         return self.cli_provider
-        
-    def cli(self,cmd,command_submit=None,bulk_lines=None):
+    ##
+    ## Execute CLI command and return a result
+    ## if list_re is None, return a string
+    ## if list_re is regular expression object, return a list of dicts (group name -> value), one dict per matched line
+    ##
+    def cli(self,cmd,command_submit=None,bulk_lines=None,list_re=None):
         #
         self.debug("cli(%s)"%cmd)
         self.cli_debug(cmd,">")
-        self.request_cli_provider()
         # Submit command
         command_submit=self.profile.command_submit if command_submit is None else command_submit
-        self.cli_provider.submit(cmd,command_submit=command_submit,bulk_lines=bulk_lines)
-        data=self.cli_queue_get()
+        if self.activator.use_canned_session:
+            data=self.activator.cli(cmd)
+        else:
+            self.request_cli_provider()
+            self.cli_provider.submit(cmd,command_submit=command_submit,bulk_lines=bulk_lines)
+            data=self.cli_queue_get()
+        # Save canned output if requested
+        if self.activator.to_save_output:
+            self.activator.save_interaction("cli",cmd,data)
+        if isinstance(data,Exception):
+            # Exception captured
+            raise data
+        # Check for syntax error
+        if self.profile.pattern_syntax_error and re.search(self.profile.pattern_syntax_error,data):
+            raise self.CLISyntaxError()
+        # Echo cancelation
         if self.strip_echo and data.lstrip().startswith(cmd):
             data=data.lstrip()
             if data.startswith(cmd+"\n"):
@@ -299,6 +329,14 @@ class Script(threading.Thread):
             else:
                 # Some switches, like ProCurve do not send \n after the echo
                 data=data[len(cmd):]
+        # Convert to list when required
+        if list_re:
+            r=[]
+            for l in data.splitlines():
+                match=list_re.match(l.strip())
+                if match:
+                    r+=[match.groupdict()]
+            data=r
         self.debug("cli(%s) returns:\n---------\n%s\n---------"%(cmd,repr(data)))
         self.cli_debug(data,"<")
         return data
@@ -374,6 +412,8 @@ class Script(threading.Thread):
             self.parent.schedule_to_save()
     
     def motd(self):
+        if self.activator.use_canned_session:
+            return self.activator.get_motd()
         if not self.cli_provider:
             self.request_cli_provider()
         return self.cli_provider.motd
@@ -571,6 +611,9 @@ class CLI(StreamFSM):
     
     def on_FAILURE_enter(self):
         self.set_patterns([])
+        if not self.is_ready:
+            self.queue.put(None)
+        self.queue.put(Script.LoginError(self.motd))
 
     def on_START_match(self,data,match):
         self.motd+=data
@@ -859,16 +902,26 @@ class SNMPProvider(object):
         self.queue=Queue.Queue(maxsize=1)
         self.getnext_socket=None
         self.community_suffix=None
+        self.to_save_output=self.script.activator.to_save_output
     
     def get(self,oid,community_suffix=None):
+        if self.script.activator.use_canned_session:
+            r=self.script.activator.snmp_get(oid)
+            if r is None:
+                raise self.TimeOutError()
+            return r
         self.community_suffix=community_suffix
         s=SNMPGetSocket(self,oid)
         try:
             r=self.queue.get(block=True)
             if r is None:
+                if self.script.activator.to_save_output:
+                    self.script.activator.save_snmp_get(oid,None)
                 raise self.TimeOutError()
         finally:
             s.close()
+        if self.to_save_output:
+            self.script.activator.save_snmp_get(oid,r)
         return r
     ##
     ## getnext generator.
@@ -877,6 +930,15 @@ class SNMPProvider(object):
     ##      ....
     ##
     def getnext(self,oid,community_suffix=None,bulk=False,min_index=None,max_index=None):
+        if self.script.activator.use_canned_session:
+            r=self.script.activator.snmp_getnext(oid)
+            if r is None:
+                raise self.TimeOutError()
+            for l in r:
+                yield l
+            raise StopIteration
+        if self.to_save_output:
+            out=[]
         self.community_suffix=community_suffix
         if self.getnext_socket:
             self.getnext_socket.close()
@@ -888,28 +950,36 @@ class SNMPProvider(object):
             r=self.queue.get(block=True)
             if r is None:
                 if self.getnext_socket.is_failed:  # Stale socket killed
+                    if self.to_save_output:
+                        self.script.activator.save_snmp_getnext(oid,None)
                     raise self.TimeOutError()
                 elif self.getnext_socket.got_result: # Stop Iteration in case of success
+                    if self.to_save_output:
+                        self.script.activator.save_snmp_getnext(oid,out)
                     raise StopIteration
                 else: # Socket closed by Timeout
+                    if self.to_save_output:
+                        self.script.activator.save_snmp_getnext(oid,None)
                     raise self.TimeOutError()
             else:
+                if self.script.activator.to_save_output:
+                    out+=[r]
                 yield r
     ##
     ## getnext wrapper
     ## returns a hash of {index,value}
     ##
-    def get_table(self,oid,community_suffix=None,bulk=False):
+    def get_table(self,oid,community_suffix=None,bulk=False,min_index=None,max_index=None):
         r={}
-        for o,v in self.getnext(oid,community_suffix=community_suffix,bulk=bulk):
+        for o,v in self.getnext(oid,community_suffix=community_suffix,bulk=bulk,min_index=min_index,max_index=max_index):
             r[int(o.split(".")[-1])]=v
         return r
     ##
     ## Generator returning a rows of two snmp tables joined by index
     ##
-    def join_tables(self,oid1,oid2,community_suffix=None,bulk=False):
-        t1=self.get_table(oid1,community_suffix=community_suffix,bulk=bulk)
-        t2=self.get_table(oid2,community_suffix=community_suffix,bulk=bulk)
+    def join_tables(self,oid1,oid2,community_suffix=None,bulk=False,min_index=None,max_index=None):
+        t1=self.get_table(oid1,community_suffix=community_suffix,bulk=bulk,min_index=min_index,max_index=max_index)
+        t2=self.get_table(oid2,community_suffix=community_suffix,bulk=bulk,min_index=min_index,max_index=max_index)
         for k1,v1 in t1.items():
             try:
                 yield (v1,t2[k1])
